#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <vector>


#define pi 3.14159265359
#define pi2 1.5707963268

using namespace std;

#define I(x,y)   (input_image[(y)*(width)+(x)])
#define O(x,y)   (output_image[(y)*(width)+(x)])

struct point
{
	int r;
	int c;
};


unsigned char* read_pgm(char* filename, int* width, int* height);
float** fmatrix(int row, int col);
int** imatrix(int row, int col);
void write_pgm( FILE* f, int width, int height, unsigned char* data );
float** pgm_mat( char* filename, int& width, int& height );
void write_ppm( FILE* f, int width, int height, unsigned char* data );
void drawcenters(FILE* f,vector <struct point> centers,float** mat,int width, int height);

int main(int argc, char **argv)
{
   float **image ;		/* image array */
   int	**accumu ;		/* accumulator array */
   int	width, height, i, j, k;	/* row and column of images */
   float  r;                      /* radius trials */
   vector <struct point> centers;
   struct point pts; 
   FILE* f;

   /*check command line arguments */   
   if (argc < 2)
   {
	  fprintf( stderr, "usage: %s inputfile radius > outputfile\n", argv[0] );
      exit(0) ;
   }


   r=atoi(argv[2]);
   image=pgm_mat(argv[1],width,height);

  /* Allocate and initialize accumulators */
   accumu = imatrix(height,width);

   for (i = 0; i < height; i++)
     for (j = 0; j < width; j++)
        accumu[i][j] = 0;
  

   //Please write your circle finding code here  

	// find the center candidates from accumulation matrix 


	// cluster the center candidates 

	//draw the centers of circles on top of input image
	drawcenters(stdout,centers,image,width,height);

}





unsigned char* read_pgm(char* filename, int* width, int* height)
{
     char buffer[80];
     int phase = 0;
     int type, maxval;
     int j;
     unsigned char* data;
     FILE* f;

     if ( strcmp( filename, "-" ) == 0 )
     {
	  f = stdin;
     }
     else
     {
	  f = fopen( filename, "rb" );
	  if ( f == NULL )
	       return NULL;
     }
     
     while ( phase < 4 )
     {
	  fgets( buffer, 80, f );
	  if ( buffer[0] == '#' ) continue;
	  switch( phase )
	  {
	     case 0: j = sscanf( buffer, "P%d %d %d %d\n", &type, width, height, &maxval ); break;
	     case 1: j = sscanf( buffer, "%d %d %d\n", width, height, &maxval ); break;
	     case 2: j = sscanf( buffer, "%d %d\n", height, &maxval ); break;
	     case 3: j = sscanf( buffer, "%d\n", &maxval ); break;
	     case 4: j = 0; break;
	  }
	  phase += j;
     }

     if ( type != 5 )
     {
	  fprintf( stderr, "bad input image format.\n" );
	  return NULL;
     }

     data = (unsigned char*)malloc ( *width * *height );
     fread( data, *width * *height, 1, f );

     fclose( f );
     
     return data;
}

/*****************************************************************************
 *      Allocate memory for float matrix.
 *****************************************************************************/
float** fmatrix(int row, int col)
{
   int i;
   float **m;

   m=new float* [row];
   for (i=0;i<row;i++)
      m[i] = new float[col];
   return m ;
}

/*****************************************************************************
 *      Allocate memory for integer matrix.
 *****************************************************************************/
int** imatrix(int row, int col)
{
   int i;
   int **m;
   m=new int* [row];
   for (i=0;i<row;i++)
      m[i] = new int[col];
   return m;
}

void write_pgm( FILE* f, int width, int height, unsigned char* data )
{
     fprintf( f, "P5\n%d %d\n%d\n", width, height, 255 );
     fwrite( data, width*height, 1, f );
     fclose( f );
}


/*****************************************************************************
 *      Read pgm image into float matrix.
 *****************************************************************************/
float** pgm_mat( char* filename, int& width, int& height )
{
	 int i,j,k;
	 FILE* f;
	 unsigned char* input_image;
	 float** input_image_double;

	 input_image=read_pgm(filename,&width,&height);
	 input_image_double=new float* [height];
	 for (i=0;i<height;i++)
	 {
		 input_image_double[i]=new float [width];
		 for (j=0;j<width;j++)
			input_image_double[i][j]=I(j,i);
	 }
	 return input_image_double;
}


void write_ppm( FILE* f, int width, int height, unsigned char* data )
{
    fprintf( f, "P6\n%d %d\n%d\n", width, height, 255 );
    fwrite( data, width*height*3, 1, f );
    fclose( f );
}

/*****************************************************************************
 *      Draw the centers of circles on top of input image
 *****************************************************************************/
void drawcenters(FILE* f,vector <struct point> centers,float** mat,int width, int height)
{
	unsigned char* output;
	int i,j,k;
	output=new unsigned char [3*width*height];
	for (i=0;i<width;i++)
		for (j=0;j<height;j++)
			for (k=0;k<3;k++)
				output[3*((j)*(width)+(i))+(k)]=(unsigned char) mat[j][i];

	for (i=0;i<centers.size();i++)
	{
		for (j=0;j<5;j++)
		{
			output[3*((centers[i].r-2+j)*width+(centers[i].c))+0]=255;
			output[3*((centers[i].r-2+j)*width+(centers[i].c))+1]=0;
			output[3*((centers[i].r-2+j)*width+(centers[i].c))+2]=0;
		}
		for (k=0;k<5;k++)
		{
			output[3*((centers[i].r)*width+(centers[i].c-2+k))+0]=255;
			output[3*((centers[i].r)*width+(centers[i].c-2+k))+1]=0;
			output[3*((centers[i].r)*width+(centers[i].c-2+k))+2]=0;
		}

	}
	write_ppm(f,width,height,output);
}